@@ -28,6 +28,7 @@ controller_params = ["--nodb", "--hwm=1", "--scheme=lru"]
 class BcbioLSFEngineSetLauncher(launcher.LSFEngineSetLauncher):
     """Custom launcher handling heterogeneous clusters on LSF.
     """
+    batch_file_name = Unicode(unicode("lsf_engine" + str(uuid.uuid4())))
     cores = traitlets.Integer(1, config=True)
     default_template = traitlets.Unicode("""#!/bin/sh
 #BSUB -q {queue}
@@ -44,6 +45,8 @@ class BcbioLSFEngineSetLauncher(launcher.LSFEngineSetLauncher):
         return super(BcbioLSFEngineSetLauncher, self).start(n)
 
 class BcbioLSFControllerLauncher(launcher.LSFControllerLauncher):
+    batch_file_name = Unicode(unicode("lsf_controller" + str(uuid.uuid4())))
+    "-".join( str(uuid.uuid4))
     default_template = traitlets.Unicode("""#!/bin/sh
 #BSUB -J bcbio-ipcontroller
 #BSUB -oo bcbio-ipcontroller.bsub.%%J
@@ -57,6 +60,7 @@ class BcbioLSFControllerLauncher(launcher.LSFControllerLauncher):
 class BcbioSGEEngineSetLauncher(launcher.SGEEngineSetLauncher):
     """Custom launcher handling heterogeneous clusters on SGE.
     """
+    batch_file_name = Unicode(unicode("sge_engine" + str(uuid.uuid4())))
     cores = traitlets.Integer(1, config=True)
     pename = traitlets.Unicode("", config=True)
     default_template = traitlets.Unicode("""#$ -V
@@ -78,6 +82,7 @@ class BcbioSGEEngineSetLauncher(launcher.SGEEngineSetLauncher):
         return super(BcbioSGEEngineSetLauncher, self).start(n)
 
 class BcbioSGEControllerLauncher(launcher.SGEControllerLauncher):
+    batch_file_name = Unicode(unicode("sge_controller" + str(uuid.uuid4())))
     default_template = traitlets.Unicode(u"""#$ -V
 #$ -S /bin/sh
 #$ -N ipcontroller
@@ -103,6 +108,7 @@ def _find_parallel_environment():
 class BcbioPBSEngineSetLauncher(launcher.PBSEngineSetLauncher):
     """Custom launcher handling heterogeneous clusters on SGE.
     """
+    batch_file_name = Unicode(unicode("pbs_engines" + str(uuid.uuid4())))
     cores = traitlets.Integer(1, config=True)
     pename = traitlets.Unicode("", config=True)
     default_template = traitlets.Unicode("""#PBS -V
@@ -122,6 +128,7 @@ class BcbioPBSEngineSetLauncher(launcher.PBSEngineSetLauncher):
 
 
 class BcbioPBSControllerLauncher(launcher.PBSControllerLauncher):
+    batch_file_name = Unicode(unicode("pbs_controller" + str(uuid.uuid4())))
     default_template = traitlets.Unicode(u"""#PBS -V
 #PBS -S /bin/sh
 #PBS -N ipcontroller
@@ -152,8 +159,8 @@ class TORQUELauncher(launcher.BatchSystemLauncher):
 class BcbioTORQUEEngineSetLauncher(TORQUELauncher, launcher.BatchClusterAppMixin):
     """Launch Engines using PBS"""
     cores = traitlets.Integer(1, config=True)
-    batch_file_name = Unicode(u'torque_engines', config=True,
-        help="batch file name for the engine(s) job.")
+    batch_file_name = Unicode(unicode("torque_engines" + str(uuid.uuid4())),
+                              config=True, help="batch file name for the engine(s) job.")
     default_template= Unicode(u"""#!/bin/sh
 #PBS -V
 #PBS -j oe
@@ -171,9 +178,9 @@ class BcbioTORQUEEngineSetLauncher(TORQUELauncher, launcher.BatchClusterAppMixin
 
 class BcbioTORQUEControllerLauncher(TORQUELauncher, launcher.BatchClusterAppMixin):
     """Launch a controller using PBS."""
+    batch_file_name = Unicode(unicode("torque_controller" + str(uuid.uuid4())),
+                              config=True, help="batch file name for the engine(s) job.")
 
-    batch_file_name = Unicode(u'torque_controller', config=True,
-        help="batch file name for the controller job.")
     default_template= Unicode("""#!/bin/sh
 #PBS -V
 #PBS -N ipcontroller
@@ -223,7 +230,7 @@ class BcbioPBSPROControllerLauncher(PBSPROLauncher, launcher.BatchClusterAppMixi
         """Start the controller by profile or profile_dir."""
         return super(BcbioPBSPROControllerLauncher, self).start(1)
 
-def _start(scheduler, profile, queue, num_jobs, cores_per_job):
+def _start(scheduler, profile, queue, num_jobs, cores_per_job, cluster_id):
     """Starts cluster from commandline.
     """
     ns = "cluster_helper.cluster"
@@ -241,20 +248,23 @@ def _start(scheduler, profile, queue, num_jobs, cores_per_job):
          "--%s.cores=%s" % (engine_class, cores_per_job),
          "--IPClusterStart.controller_launcher_class=%s.%s" % (ns, controller_class),
          "--IPClusterStart.engine_launcher_class=%s.%s" % (ns, engine_class),
-         "--%sLauncher.queue='%s'" % (scheduler, queue)
+         "--%sLauncher.queue='%s'" % (scheduler, queue),
+         "--cluster-id=%s" % (cluster_id)
          ]
     if scheduler in ["SGE"]:
         args += ["--%s.pename=%s" % (engine_class, _find_parallel_environment())]
 
     subprocess.check_call(args)
+    return cluster_id
 
-def _stop(profile):
+def _stop(profile, cluster_id):
     subprocess.check_call(launcher.ipcluster_cmd_argv +
-                          ["stop", "--profile=%s" % profile])
+                          ["stop", "--profile=%s" % profile,
+                           "--cluster-id=%s" % cluster_id])
 
-def _is_up(profile, n):
+def _is_up(url_file, n):
     try:
-        client = Client(profile=profile)
+        client = Client(url_file)
         up = len(client.ids)
         client.close()
     except IOError, msg:
@@ -280,9 +290,13 @@ def cluster_view(scheduler, queue, num_jobs, cores_per_job=1, profile=None):
     else:
         has_throwaway = False
     num_tries = 0
+
+    cluster_id = str(uuid.uuid4())
+    url_file = get_url_file(profile, cluster_id)
+    #cluster_id = ""
     while 1:
         try:
-            _start(scheduler, profile, queue, num_jobs, cores_per_job)
+            _start(scheduler, profile, queue, num_jobs, cores_per_job, cluster_id)
             break
         except subprocess.CalledProcessError:
             if num_tries > max_tries:
@@ -292,23 +306,17 @@ def cluster_view(scheduler, queue, num_jobs, cores_per_job=1, profile=None):
     try:
         client = None
         slept = 0
-        while not _is_up(profile, num_jobs):
+        while not _is_up(url_file, num_jobs):
             time.sleep(delay)
             slept += delay
             if slept > max_delay:
                 raise IOError("Cluster startup timed out.")
-        #client = Client(profile=profile, cluster_id=cluster_id)
-        client = Client(profile=profile)
-        # push config to all engines and force them to set up logging
-        #client[:]['config'] = config
-        #client[:].execute('from bcbio.log import setup_logging')
-        #client[:].execute('setup_logging(config)')
-        #client[:].execute('from bcbio.log import logger')
+        client = Client(url_file)
         yield _get_balanced_blocked_view(client)
     finally:
         if client:
             client.close()
-        _stop(profile)
+        _stop(profile, cluster_id)
         if has_throwaway:
             delete_profile(profile)
 
@@ -325,6 +333,20 @@ def create_throwaway_profile():
     subprocess.check_call(cmd, shell=True)
     return profile
 
+
+def get_ipython_dir(profile):
+    proc = subprocess.Popen("ipython locate", stdout=subprocess.PIPE, shell=True)
+    ipython_dir = proc.stdout.read().strip()
+    profile_dir = "profile_{0}".format(profile)
+    return os.path.join(ipython_dir, profile_dir)
+
+def get_url_file(profile, cluster_id):
+    ipython_dir = get_ipython_dir(profile)
+    security_dir = os.path.join(ipython_dir, "security")
+    url_file = "ipcontroller-{0}-client.json".format(cluster_id)
+    return os.path.join(security_dir, url_file)
+
+
 def delete_profile(profile):
     MAX_TRIES = 10
     proc = subprocess.Popen("ipython locate", stdout=subprocess.PIPE, shell=True)
